@@ -2,7 +2,7 @@
   Daemonshogi -- a GTK+ based, simple shogi (Japanese chess) program.
 
   Copyright (C) Masahiko Tokita    2002-2005,2009 
-  Copyright (C) Hisashi Horikawa   2008-2009
+  Copyright (C) Hisashi Horikawa   2008-2010
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -65,40 +65,8 @@ G_END_DECLS
 #include "si/ui.h"
 
 
-/** ログウィンドウに表示 */
-void network_log_append_message(const char* format, ...)
-{
-  char buf[1000];
-  va_list ap;
-  // GtkTextBuffer* textbuf;
-  // GtkWidget* network_textview;
-
-  va_start(ap, format);
-  vsprintf(buf, format, ap);
-  va_end(ap);
-
-  printf("log: %s\n", buf); // DEBUG
-#if 0
-  gdk_threads_enter();
-  
-  network_textview = lookup_widget(g_canvas->network_log_window, 
-				   "network_textview");
-  assert(network_textview);
-
-  // TODO: きちんと末尾に追加する。
-  textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(network_textview));
-  gtk_text_buffer_insert_at_cursor(textbuf, buf, -1);
-
-  gdk_threads_leave();
-#endif // 0
-}
-
-
-/** 受信データを行に分ける */
-typedef list<string> RecvList;
-
 /** 行に分けた受信データ */
-static RecvList recv_list;
+static RecvBuffer recv_buffer;
 
 static int wait_state = 0;
 static gint io_watch = 0;
@@ -108,45 +76,39 @@ static int server_fd = -1;
 /** 受信した指し手 */
 struct NetTe 
 {
-  /** 投了、時間切れなど */
-  int special;
-
-  /** 指し手 */
-  TE te;
+  /** 指し手. 投了, 千日手なども */
+  DTe te;
 
   /** 消費秒数 */
   int sec;
 
-  NetTe(): special(SI_NORMAL), sec(0) { memset(&te, 0, sizeof(TE)); }
+  NetTe(): sec(0) { }
 };
 
 typedef list<NetTe> NetTeList;
 static NetTeList net_te_list;
 
+extern GtkWidget* connect_status_dialog;
+
 /** サーバとの接続を切断。 */
 void disconnect_server()
 {
+  gtk_widget_hide(connect_status_dialog);
+
   if (server_fd < 0)
     return;
 
-  recv_list.clear();
+  recv_buffer.clear();
   wait_state = 0;
   net_te_list.clear();
 
-  // g_source_remove(idle_id);
-  // idle_id = 0;
-
   g_source_remove(io_watch);
   io_watch = 0;
 
   closesocket(server_fd);
   server_fd = -1;
 
-  // GtkWidget* item = lookup_widget(g_canvas->window, "connect_server");
-  // assert(item);
-  // gtk_widget_set_sensitive(item, TRUE);
-
-  daemon_canvas_change_mode(g_canvas, D_CANVAS_MODE_BOOK);
+  // daemon_canvas_change_mode(g_canvas, D_CANVAS_MODE_BOOK);
 }
 
 
@@ -164,20 +126,49 @@ static void csa_send(const char* line)
 
 static string username;
 
-// extern char net_playername[2][100];
-// extern char net_my_turn;
-// static int game_total_time;
-
 NetGameInfo game_info;
 
 
 /**
  * 持ち時間をパース.
- * \todo impl.
  */
-static void parse_time(RecvList::iterator it)
+static bool parse_time()
 {
-  game_info.total_time = 0;
+  while ( recv_buffer.recv_list.size() > 0 ) {
+    string line = recv_buffer.recv_list.front();
+    recv_buffer.recv_list.pop_front();
+
+    if (line.find("END Time") == 0) {
+      wait_state = 2;
+      return true;
+    }
+    
+    string name, value;
+    int pos;
+
+    if ( (pos = line.find(":")) > 0 ) {
+      name = line.substr(0, pos);
+      value = line.substr(pos + 1);
+    }
+    else
+      name = "";
+
+    if (name == "Time_Unit") {
+      if ( value != "1sec" ) {
+	si_abort( "not support except 'one second.' value = %s\n", 
+		  value.c_str() );
+      }
+      // TODO: impl.
+    }
+    else if (name == "Total_Time")
+      game_info.total_time = atoi(value.c_str());
+    else if (name == "Byoyomi")
+      ;
+    else if (name == "Least_Time_Per_Move")
+      ;
+  }
+
+  return false; // 引き続き呼び出してもらう
 }
 
 
@@ -192,14 +183,17 @@ static bool state_2_read()
   printf("%s\n", __func__); // DEBUG
 
   bool finished = false;
-  string name, value;
-  int pos;
-  
-  RecvList::iterator it;
-  for (it = recv_list.begin(); it != recv_list.end(); it++) {
-    if ( (pos = it->find(":")) > 0 ) {
-      name = it->substr(0, pos);
-      value = it->substr(pos + 1);
+
+  while ( recv_buffer.recv_list.size() > 0 ) {
+    string line = recv_buffer.recv_list.front();
+    recv_buffer.recv_list.pop_front();
+
+    string name, value;
+    int pos;
+
+    if ( (pos = line.find(":")) > 0 ) {
+      name = line.substr(0, pos);
+      value = line.substr(pos + 1);
     }
     else
       name = "";
@@ -214,13 +208,14 @@ static bool state_2_read()
       game_info.my_turn = value[0];
       assert( game_info.my_turn == '+' || game_info.my_turn == '-' );
     }
-    else if (it->find("BEGIN Time") == 0) {
-      parse_time(it);
+    else if ( line.find("BEGIN Time") == 0) {
+      wait_state = 21;
+      return false;
     }
-    else if (it->find("BEGIN Position") == 0) {
-      // TODO:
+    else if ( line.find("BEGIN Position") == 0) {
+      // TODO: impl.
     }
-    else if (it->find("END Game_Summary") == 0) {
+    else if ( line.find("END Game_Summary") == 0) {
       finished = true;
       break;
     }
@@ -232,8 +227,6 @@ static bool state_2_read()
   }
 
   printf("receive summary.\n"); // DEBUG
-
-  recv_list.erase(recv_list.begin(), ++it);
   return true;
 }
 
@@ -241,10 +234,11 @@ static bool state_2_read()
 /** ゲーム開始 or 対戦相手からreject */
 static bool state_3_read()
 {
-  printf("%s: line = %s\n", __func__, recv_list.front().c_str()); // DEBUG
+  printf("%s: line = %s\n", __func__, 
+	 recv_buffer.recv_list.front().c_str()); // DEBUG
 
-  string line = recv_list.front(); 
-  recv_list.pop_front();
+  string line = recv_buffer.recv_list.front(); 
+  recv_buffer.recv_list.pop_front();
   if (line.find("START:", 0) == string::npos) {
     // reject ?
     // network_log_append_message("reject game.\n");
@@ -264,73 +258,182 @@ const char csa_koma_str[][3] = {
 };
 
 /** 手を作る. daemon_record_load_csa_te() を参考に。 */
-static void parse_te(const char* line, TE* te, int* sec) 
+static void parse_te( const string& line, NetTe* net_te )
 {
-  char c;
-  char koma_str[3];
-  int p;
+  assert( net_te );
 
-  assert(te);
-  assert(sec);
+  int idx = 0;
 
-  // xyが逆
-  c = line[0];
-  te->fm = ((line[2] - 0x30) << 4) + line[1] - 0x30;
-  te->to = ((line[4] - 0x30) << 4) + line[3] - 0x30;
-  sscanf(line + 5, "%2s,T%d", koma_str, sec);
-
-  for (p = 1; p < 0xf; p++) {
-    if (strcmp(koma_str, csa_koma_str[p]) == 0)
-      break;
+  if ( !line.find("%TORYO") ) {
+    net_te->te.clear();
+    net_te->te.special = DTe::RESIGN;
+    idx += strlen("%TORYO");
   }
-
-  if (te->fm == 0) {
-    // 駒打ち
-    te->nari = 0;
-    te->uti = p;
+  else if ( !line.find("%KACHI") ) {
+    net_te->te.clear();
+    net_te->te.special = DTe::KACHI;
+    idx += strlen("%KACHI");
   }
   else {
-    // 移動
-    te->uti = 0;
-    int pf = daemon_dboard_get_board(&g_canvas->board, 
-				     te->fm & 0xf, 
-				     te->fm >> 4) & 0xf;
-    te->nari = pf <= 8 && p >= 9 ? 1 : 0;
+    // 通常の指し手
+    assert( line[0] == '+' || line[0] == '-' );
+
+    char koma_str[3];
+    PieceKind p;
+
+    net_te->te.clear();
+    net_te->te.special = DTe::NORMAL_MOVE;
+    // xyが逆
+    Xy fm = ((line[2] - 0x30) << 4) + line[1] - 0x30;
+    Xy to = ((line[4] - 0x30) << 4) + line[3] - 0x30;
+
+    sscanf(line.c_str() + 5, "%2s", koma_str );
+    for (p = 1; p < 0xf; p++) {
+      if (strcmp(koma_str, csa_koma_str[p]) == 0)
+	break;
+    }
+
+    if ( fm == 0 ) {
+      // 駒打ち
+      net_te->te.to = to; net_te->te.uti = p;
+    }
+    else {
+      // 移動
+      PieceKind pf = daemon_dboard_get_board(&g_canvas->board, 
+				     fm & 0xf, 
+				     fm >> 4) & 0xf;
+      net_te->te.fm = fm;
+      net_te->te.to = to;
+      net_te->te.nari = pf <= 8 && p >= 9 ? true : false;
+      net_te->te.tori = g_canvas->board.board[to];
+    }
+
+    idx += 7;
   }
 
-  printf("%s: from = %x, to = %x\n", __func__, te->fm, te->to);
+  // 消費時間
+  net_te->sec = 1; // 0 はない
+  sscanf( line.c_str() + idx, ",T%d", &net_te->sec );
+
+  printf("%s: parsed = %s, %d\n", 
+	 __func__, net_te->te.to_str(g_canvas->board).c_str(), net_te->sec);
 }
 
 
-/** 相手の手を受信 */
+static NetTe pending_move;
+
+/** 相手の手または自分の手番で時間切れを受信 */
 static bool state_4_read()
 {
-  printf("%s: front = '%s'\n", __func__, recv_list.front().c_str()); // DEBUG
+  printf("%s: front = '%s'\n", __func__, 
+	 recv_buffer.recv_list.front().c_str()); // DEBUG
 
-  string line = recv_list.front();
-  recv_list.pop_front();
+  string line = recv_buffer.recv_list.front();
+  recv_buffer.recv_list.pop_front();
 
-  // TODO: 時間切れなどの検査
-
-  // 勝ったとき
-  // %TORYO
-  // #RESIGN
-  // #WIN
   NetTe net_te;
-  if (!line.find("%TORYO", 0))
-    net_te.special = SI_TORYO;
-  else if (line == "#RESIGN") 
-    net_te.special = -1;
-  else if (line == "#WIN")
-    net_te.special = -2;
-  else if (line == "#LOSE")
-    net_te.special = -3;
+
+  if (line == "#JISHOGI") {
+    // プロトコルv1.1.2では「日本将棋連盟ルール」とだけ書いている.
+    // 24点法だと思うが、そうすると引き分けの場合がある
+    //
+    // http://www.computer-shogi.org/wcsc20/rule.html
+    // によれば, プロトコルを無視して, 27点法によるみたい
+    //   → 引き分けはない
+    assert( pending_move.te.special == DTe::KACHI );
+  }
+  else if (line == "#SENNICHITE") {
+    // 連続王手でない千日手. 指してもよいがそこで終了. これだけ扱いが違う
+    // クライアントでも千日手チェックするか, 非同期で受信しないといけない
+    //   +6978KI,T2
+    //   #SENNICHITE
+    //   #DRAW
+    assert( pending_move.te.special == DTe::SENNICHITE );
+  }
+  else if (line == "#OUTE_SENNICHITE") {
+    // 連続王手の千日手
+    // これもクライアントでのチェックが必要
+    //   -8493OU,T2
+    //   #OUTE_SENNICHITE
+    //   #WIN
+    assert( pending_move.te.special == DTe::ILLEGAL );
+  }
+  else if (line == "#ILLEGAL_MOVE") {
+    // これもクライアントでのチェックが必要
+    //   +0031FU,T1
+    //   #ILLEGAL_MOVE
+    //   #WIN
+    assert( pending_move.te.special == DTe::ILLEGAL || 
+	    pending_move.te.special == DTe::KACHI );
+
+    pending_move.te.special = DTe::ILLEGAL; // 不正な入玉勝利宣言
+  }
+  else if (line == "#TIME_UP") {
+    // 時間切れ. 1行目で判断できる
+    // #TIME_UP
+    // #WIN
+    pending_move.te.special = DTe::TIME_OUT;
+  }
+  else if (line == "#CHUDAN") {
+    // サーバからの中断指示
+    // クライアントからの中断要請もできるが, プロトコルv1.1.2では何が起こるか不明
+    gmSetGameStatus( &g_game, SI_CHUDAN_REMOTE );
+  }
+  else if ( line == "#RESIGN" ) {
+    assert( pending_move.te.special == DTe::RESIGN );
+  }
+  else if ( line == "#WIN" || line == "#LOSE" || line == "#DRAW" ) {
+    assert( pending_move.te.special );
+
+    if ( pending_move.te.special == DTe::TIME_OUT )
+      gmSetGameStatus( &g_game, SI_TIMEOUT );
+    else {
+      net_te_list.push_back( pending_move );
+      printf("%s: add net_te = %s\n", __func__, 
+	   pending_move.te.to_str(g_canvas->board).c_str() ); // DEBUG
+    }
+  }
   else {
-    net_te.special = SI_NORMAL;
-    parse_te(line.c_str(), &net_te.te, &net_te.sec);
+    // 通常手, 投了, 勝利宣言
+    parse_te( line, &net_te );
+    bool forbidden = false;
+    const BOARD* bo = &g_canvas->board;
+
+    if (net_te.te.special == DTe::RESIGN) {
+      // 勝ったとき. #RESIGN, #WINは無視してよい
+      //   %TORYO.T4
+      //   #RESIGN
+      //   #WIN
+      pending_move = net_te;
+    }
+    else if (net_te.te.special == DTe::KACHI) {
+      // 勝利宣言. クライアントでチェックするか2行目を待つか
+      // shogi-server では #DRAW を返すことはないようだが?
+      //   %KACHI,T8
+      //   #JISHOGI
+      //   #WIN
+      // または
+      //   %KACHI,T8
+      //   #ILLEGAL_MOVE
+      //   #WIN
+      pending_move = net_te;
+    }
+    else if ( !te_is_valid(bo, net_te.te.pack()) ) {
+      pending_move = net_te;
+      pending_move.te.special = DTe::ILLEGAL;
+      // #ILLEGAL_MOVE を待つ
+    }
+    else if ( te_is_sennichite(bo, net_te.te.pack(), &forbidden) ) {
+      pending_move = net_te;
+      pending_move.te.special = forbidden ? DTe::ILLEGAL : DTe::SENNICHITE;
+      // #OUTE_SENNICHITE または #SENNICHITE を待つ
+    }
+    else {
+      net_te.te.special = DTe::NORMAL_MOVE;
+      net_te_list.push_back(net_te);
+    }
   }
 
-  net_te_list.push_back(net_te);
   return true;
 }
 
@@ -339,59 +442,24 @@ static bool state_4_read()
  * ネットワーク経由の手を格納.
  * ブロックする 
  */
-INPUTSTATUS daemon_input_next_network_impl(BOARD* bo, TE* te)
+INPUTSTATUS daemon_input_next_network_impl( const BOARD* bo, 
+					    DTe* te, int* elapsed_time )
 {
   // 手が設定されるまで待つ
   while (net_te_list.size() == 0) {
-    pending_loop();
-    usleep(50);
+    // 時間切れ, 中断
+    if ( gmGetGameStatus(&g_game) != 0 )
+      return gmGetGameStatus(&g_game);
+
+    processing_pending( true );
   }
 
   NetTe net_te = net_te_list.front();
   net_te_list.pop_front();
 
-  if (net_te.special != SI_NORMAL) {
-    // 中断または投了
-    disconnect_server(); // TODO: もう少し丁寧に
-    return (INPUTSTATUS) net_te.special;
-  }
-  else {
-    *te = net_te.te;
-    return SI_NORMAL;
-  }
-}
-
-
-static string sock_buffer;
-
-/** サーバから読み込んで、行を切り出す.
-  データが届いていないときはブロックする
-  @return エラーまたは接続が切れたとき 0 */
-static int read_from_server()
-{
-  char buf[1000];
-  int r = ::recv(server_fd, buf, sizeof(buf) - 1, 0);
-  if (r < 0) {
-    // エラー発生
-    printf("%s: socket recv error.\n", __func__); // DEBUG
-    return 0;
-  }
-  else if (r == 0) // 接続先が閉じた
-    return 0;
-
-  buf[r] = '\0';
-  sock_buffer += buf;
-
-  // 行を切り出す. 末尾の改行は削除する
-  int idx;
-  while ((idx = sock_buffer.find("\n", 0)) != string::npos) {
-    string line = string(sock_buffer, 0, idx);
-    printf("recv: %s\n", line.c_str()); // DEBUG
-    recv_list.push_back(line);
-    sock_buffer.erase(0, idx + 1);
-  }
-
-  return 1;
+  *te = net_te.te;
+  *elapsed_time = net_te.sec;
+  return SI_NORMAL;
 }
 
 
@@ -399,29 +467,31 @@ static int read_from_server()
 static void state_1_read()
 {
   string s = "LOGIN:" + username + " OK";
-  if (recv_list.front() == s) {
+  if ( recv_buffer.recv_list.front() == s) {
     // network_log_append_message(_("%s login ok.\n"), arg->username.c_str());
     // arg->status_queue.push_back(_("login ok."));
-    recv_list.pop_front();
+    recv_buffer.recv_list.pop_front();
     wait_state = 2;
   }
   else {
-    // network_log_append_message(_("'%s' login failed.\n"), arg->username.c_str());
-    // arg->status_queue.push_back(_("reject login."));
     disconnect_server();
+    daemon_messagebox(g_canvas, _("login failed."), GTK_MESSAGE_ERROR);
+    g_signal_emit_by_name(
+		  G_OBJECT(lookup_widget(g_canvas->window, "connect_server")),
+		  "activate", NULL);
   }
 }
 
 
-extern void daemon_start_game_via_network();
-extern GtkWidget* connect_status_dialog;
-
+#if 0
 struct SocketHook {
   bool (*on_game_proposal)();
   
   void (*on_game_start)();
  
 };
+#endif // 0
+
 
 /** 
  * ソケットのコールバック関数.
@@ -431,15 +501,15 @@ static gboolean on_socket_read( GIOChannel* source,
                                 GIOCondition condition,
                                 gpointer data )
 {
-  if (!read_from_server()) {
-    network_log_append_message(_("server shutdown or an error occured.\n"));
+  if ( !recv_buffer.read_from_server(server_fd) ) {
+    // network_log_append_message(_("server shutdown or an error occured.\n"));
     disconnect_server();
     return FALSE;
   }
 
   GtkWidget* dialog;
 
-  while (recv_list.size() > 0) {
+  while ( recv_buffer.recv_list.size() > 0) {
     switch (wait_state) {
     case 1:
       // ログイン待ち
@@ -447,49 +517,57 @@ static gboolean on_socket_read( GIOChannel* source,
       break;
     case 2:
       // ゲーム情報待ち
-      if (!state_2_read())
-        return TRUE; // データが一部届いていない. 届くまで待つ
+      if (!state_2_read()) {
+	if (wait_state == 2)
+	  return TRUE; // データが一部届いていない. 届くまで待つ
+	break;
+      }
 
-      gtk_widget_destroy(connect_status_dialog);
-      connect_status_dialog = NULL;
+      gtk_widget_hide(connect_status_dialog);
 
       // 受けるか拒否するかだけ
       dialog = create_network_game_dialog();
       if ( gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK ) {
+        csa_send_logout();
         disconnect_server();
-        gtk_widget_destroy(dialog);
+	gtk_widget_hide(dialog);
         return FALSE;
       }
-      gtk_widget_destroy(dialog);
+      gtk_widget_hide(dialog);
 
       csa_send("AGREE\n");
       wait_state = 3;
       break;
+    case 21:
+      // ゲーム情報 -> 持ち時間
+      if ( !parse_time() )
+	return TRUE; // 一部届いていない
+      break;
     case 3:
       // ゲーム開始待ち
       if (state_3_read()) {
 	g_signal_emit_by_name(G_OBJECT(g_canvas->window), 
 			      "notify::startgame", NULL);
 	return FALSE; // watchをいったん終了
-        // daemon_start_game_via_network(); // ゲーム開始
       }
       else {
-        disconnect_server(); // 対戦相手からreject
+        // 対戦相手からreject
+        csa_send_logout();
+        disconnect_server(); 
         daemon_messagebox(g_canvas, _("reject from the opponent"), 
 			  GTK_MESSAGE_INFO);
         return FALSE;
       }
       break;
     case 4:
-      // 相手の手を待つ
+      // 相手の手または自分の手番で時間切れ
       state_4_read();
       break;
     default:
       printf("wait_state = %d\n", wait_state); // DEBUG
-      assert(0);
+      abort();
       break;
     }
-    
   }
   return TRUE; // 引き続き呼び出してもらう
 }
@@ -572,9 +650,10 @@ int connect_to_server(const string& hostname, const string& service)
 // CSAプロトコル
 
 
-/** ログイン要求を送信する.
-    @return   1 成功 
-*/
+/**
+ * ログイン要求を送信する.
+ * @return   1 成功 
+ */
 static int csa_send_login(const string& username, const string& password)
 {
   string buf = "LOGIN " + username + " " + password + "\n";
@@ -583,6 +662,24 @@ static int csa_send_login(const string& username, const string& password)
 }
 
 
+/**
+ * ログアウト要求を送信する
+ */
+void csa_send_logout()
+{
+  csa_send("LOGOUT\n");
+
+// TODO:  "LOGOUT:completed" を待つ
+}
+
+
+int csa_send_chudan_request()
+{
+  csa_send("%CHUDAN\n");
+  return 1;
+}
+
+
 /** CSAサーバに接続してログインする. ワーカースレッドとして呼び出す */
 void* csa_connect_thread_main(void* arg_)
 {
@@ -610,9 +707,10 @@ void add_watch_socket()
                              GIOCondition( G_IO_IN | G_IO_HUP | G_IO_ERR ),
                              on_socket_read,
                              NULL );
+  assert( io_watch > 0 );
   g_io_channel_unref( channel );
 
-  printf("io_watch = %d\n", io_watch); // DEBUG
+  // printf("io_watch = %d\n", io_watch); // DEBUG
 }
 
 
@@ -632,25 +730,32 @@ void csa_login_and_wait_socket(const char* username_, const char* password)
  * 指し手を送信.
  * daemon_record_output_csa_te() を参考に。 
  */
-int csa_send_move(const DBoard* board, int is_sente, const TE* te)
+int csa_send_move(const BOARD* board, int is_sente, const DTe& te)
 {
+  if (te.special == DTe::RESIGN) {
+    csa_send("%TORYO\n");
+    return 1;
+  }
+
+  // 通常の指し手
   char buf[100];
-  if ( te_is_put(te) ) {
+  if ( !te.fm ) {
     // 打ち
     sprintf(buf, "%c%d%d%d%d%s\n",
 	    is_sente ? '+' : '-',
 	    0, 0, 
-	    te->to & 0xf, te->to >> 4,
-	    csa_koma_str[te->uti]);
+	    te.to & 0xf, te.to >> 4,
+	    csa_koma_str[te.uti]);
   }
   else {
     // 移動
-    int p = daemon_dboard_get_board(board, te->fm & 0xf, te->fm >> 4);
-    if (te->nari) p += 8;
+    PieceKind p = daemon_dboard_get_board(board, te.fm & 0xf, te.fm >> 4);
+    if ( te.nari ) p += 8;
+
     sprintf(buf, "%c%d%d%d%d%s\n",
 	    is_sente ? '+' : '-',
-	    te->fm & 0xf, te->fm >> 4,
-	    te->to & 0xf, te->to >> 4,
+	    te.fm & 0xf, te.fm >> 4,
+	    te.to & 0xf, te.to >> 4,
 	    csa_koma_str[p & 0xf]);
   }
   csa_send(buf);
@@ -659,29 +764,25 @@ int csa_send_move(const DBoard* board, int is_sente, const TE* te)
 }
 
 
-/** 投了メッセージを送信 */
-int csa_send_toryo()
-{
-  csa_send("%TORYO\n");
-  return 1;
-}
-
-
-/** 手を送信した結果（経過秒数など）を得る.
-    ブロックする。 */
-int csa_wait_for_result(int* elapsed_time, int* minmax_return)
+/**
+ * 手を送信した結果 (経過秒数など) を得る.
+ * ブロックする。
+ */
+INPUTSTATUS csa_wait_for_result( DTe* te, int* elapsed_time )
 {
   printf("%s\n", __func__); // DEBUG
 
   while (net_te_list.size() == 0)
-    pending_loop();
+    processing_pending( true );
 
   NetTe net_te = net_te_list.front();
   net_te_list.pop_front();
 
-  *minmax_return = net_te.special;
+  // 千日手のことがある
+  *te = net_te.te;
   *elapsed_time = net_te.sec;
   printf("my time = %d\n", *elapsed_time); // DEBUG
 
-  return 1;
+  // きわどいタイミングで時間切れのことがある
+  return gmGetGameStatus(&g_game);
 }
