@@ -2,7 +2,7 @@
   Daemonshogi -- a GTK+ based, simple shogi (Japanese chess) program.
 
   Copyright (C) Masahiko Tokita    2002-2005,2009 
-  Copyright (C) Hisashi Horikawa   2008-2009
+  Copyright (C) Hisashi Horikawa   2008-2010
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,28 +24,42 @@
 #include <string.h>
 #include <assert.h>
 #include <tcutil.h>  // TCMAP
+#include <tchdb.h>
 #include <stddef.h> // offsetof
 #include "si.h"
+#include "si-think.h"
 #include "ui.h"
+#include "taboo_list.h"
+#include <iostream>
+using namespace std;
 
-#include "hashval.h"
 
+// #define HASH_ON_FILE 1
+#undef HASH_ON_FILE
+
+
+#define TTABLE_COUNT 6
 
 /**
- * 置換表 transposition table.
+ * 置換表 transposition table. [0], [1] は詰め将棋用、[2], [3] は必至探索用
  * オンメモリは LRUアルゴリズムで表から追い出す.
  * TCMAPは末尾に追加される。先頭を追い出すだけでいい。
  *
  * \todo 分散化するときに Tokyo Tyrantなどに移行
  * http://alpha.mixi.co.jp/blog/?p=166
  */
-static TCMAP* ttable_mate[2] = {NULL, NULL};
+static TCMAP* ttable_mate[ TTABLE_COUNT ] = {NULL, NULL, NULL, NULL, NULL, NULL};
 
+#ifdef HASH_ON_FILE
+static TCHDB* ttable = NULL;
+#else
 /** 指し将棋用 */
 static TCMAP* ttable = NULL;
+#endif 
+
 
 /** 1レコード48バイトとして、データで30Mバイト. */
-#define TTABLE_COUNT 625000
+#define TTABLE_ENT_COUNT 625000
 
 
 /**
@@ -53,6 +67,11 @@ static TCMAP* ttable = NULL;
  */
 void newHASH()
 {
+#ifndef NDEBUG
+  assert( (sizeof(EntVariant) % 8) == 0 );
+  assert( sizeof(EntVariant) == HASH_ENT_SIZE );
+  assert( (sizeof(HASH) % 8) == 0 );
+
   if ( sizeof(HASH) != HASH_HEADER_SIZE + HASH_ENT_SIZE ||
        offsetof(HASH, ent) != HASH_HEADER_SIZE ) {
     printf( "entry size: expected = %d, actual = %ld\n",
@@ -61,13 +80,23 @@ void newHASH()
             HASH_HEADER_SIZE, offsetof(HASH, ent) );
     abort();
   }
+#endif // !NDEBUG
 
   if (ttable)
     freeHASH();
 
-  ttable_mate[0] = tcmapnew();
-  ttable_mate[1] = tcmapnew();
+  for (int i = 0; i < TTABLE_COUNT; i++) 
+    ttable_mate[ i ] = tcmapnew();
+
+#ifdef HASH_ON_FILE
+  ttable = tchdbnew();
+  bool r = tchdbsetcache( ttable, TTABLE_ENT_COUNT );
+  assert( r );
+  r = tchdbopen(ttable, "/tmp/daemonshogi.bin", HDBOWRITER | HDBOCREAT );
+  assert( r );
+#else
   ttable = tcmapnew();
+#endif
 }
 
 
@@ -77,19 +106,22 @@ void newHASH()
 void freeHASH()
 {
   if (ttable) {
-    tcmapdel(ttable_mate[0]);
-    tcmapdel(ttable_mate[1]);
-    tcmapdel(ttable);
+    for (int i = 0; i < TTABLE_COUNT; i++) {
+      tcmapdel( ttable_mate[i] );
+      ttable_mate[ i ] = NULL;
+    }
 
-    ttable_mate[0] = NULL;
-    ttable_mate[1] = NULL;
+#ifdef HASH_ON_FILE
+    tchdbclose(ttable);
+    tchdbdel(ttable);
+#else
+    tcmapdel(ttable);
+#endif
     ttable = NULL;
   }
 }
 
 
-extern const int arr_xy[];
-
 /**
  * BOARD からハッシュ値を生成する。持ち駒は考慮しない。
  * @param bo 対象のBOARD
@@ -106,18 +138,12 @@ uint64_t hsGetHashKey(const BOARD* bo)
 
   key = 0;
 
-  if ( bo->next == SENTE ) {
-    for ( i=1; i<=81; i++ ) {
-      key ^= hashval[bo->board[ arr_xy[ i ] ]][ arr_xy[ i ] ];
-    }
-  } else {
-    /* 後手番の場合のハッシュキーはビット反転する */
-    for ( i=1; i<=81; i++ ) {
-      key ^= ( ~ hashval[bo->board[ arr_xy[ i ] ]][ arr_xy[ i ] ] );
-    }
+  for ( i = 1; i <= 81; i++ ) {
+    // if ( bo->board[arr_xy[i]] )
+    key ^= hashval[bo->board[ arr_xy[ i ] ]][ arr_xy[ i ] ];
   }
 
-  return key;
+  return bo->next == SENTE ? key : ~key;
 }
 
 
@@ -148,24 +174,46 @@ static void hsSetPiece(EntVariant* hs, const BOARD* bo)
 }
 
 
+#ifdef USE_PROOF_PIECES
+/** 呼び出した側で boSetToBOARD() を呼び出すこと */
+static void unpack_inhand(const EntVariant* hs, BOARD* bo)
+{
+  int32_t p = hs->packed_inhand[0];
+  int32_t q = hs->packed_inhand[1];
+
+  bo->inhand[ SENTE ][ FU ] = (p & 0x1f);
+  bo->inhand[ SENTE ][ KYO ] = ((p >> 5) & 0x7);
+  bo->inhand[ SENTE ][ KEI ] = ((p >> 8) & 0x7);
+  bo->inhand[ SENTE ][ GIN ] = ((p >> 11) & 0x7);
+  bo->inhand[ SENTE ][ KIN ] = ((p >> 14) & 0x7);
+  bo->inhand[ SENTE ][ KAKU ] = ((p >> 17) & 0x3);
+  bo->inhand[ SENTE ][ HISHA ] = ((p >> 19) & 0x3);
+
+  bo->inhand[ GOTE ][ FU ] = (q & 0x1f);
+  bo->inhand[ GOTE ][ KYO ] = ((q >> 5) & 0x7);
+  bo->inhand[ GOTE ][ KEI ] = ((q >> 8) & 0x7);
+  bo->inhand[ GOTE ][ GIN ] = ((q >> 11) & 0x7);
+  bo->inhand[ GOTE ][ KIN ] = ((q >> 14) & 0x7);
+  bo->inhand[ GOTE ][ KAKU ] = ((q >> 17) & 0x3);
+  bo->inhand[ GOTE ][ HISHA ] = ((q >> 19) & 0x3);
+}
+#endif // USE_PROOF_PIECES
+
+
 static EntVariant make_v(const BOARD* bo, int32_t phi, int32_t delta,
-                         int te_count, const TE* te)
+                         int te_count, const TE& te)
 {
   EntVariant v;
 
   hsSetPiece(&v, bo);
   v.phi = phi; v.delta = delta;
   v.te_count = te_count;
-  v.best_move = te ? te_pack(te) : 0;
+  v.best_move = te_pack(te);
 
   return v;
 }
 
 
-extern long count_entry_hash;
-extern long count_crash2;
-
-
 /**
  * 持駒を比較する。
  * @param hs 対象のHASH
@@ -176,8 +224,8 @@ static int hsCmpPiece(const EntVariant* hs, const BOARD* bo)
 {
   int next = bo->next;
   int oppo = 1 - bo->next;
-  uint32_t p = hs->packed_inhand[ next ];
-  uint32_t q = hs->packed_inhand[ oppo ];
+  int32_t p = hs->packed_inhand[ next ];
+  int32_t q = hs->packed_inhand[ oppo ];
 
   if ( bo->inhand[ next ][ FU ] != (p & 0x1f) ||
        bo->inhand[ next ][ KYO ] != ((p >> 5) & 0x7) ||
@@ -185,7 +233,7 @@ static int hsCmpPiece(const EntVariant* hs, const BOARD* bo)
        bo->inhand[ next ][ GIN ] != ((p >> 11) & 0x7) ||
        bo->inhand[ next ][ KIN ] != ((p >> 14) & 0x7) ||
        bo->inhand[ next ][ KAKU ] != ((p >> 17) & 0x3) ||
-       bo->inhand[ next ][ HISYA ] != ((p >> 19) & 0x3) ||
+       bo->inhand[ next ][ HISHA ] != ((p >> 19) & 0x3) ||
 
        bo->inhand[ oppo ][ FU ] != (q & 0x1f) ||
        bo->inhand[ oppo ][ KYO ] != ((q >> 5) & 0x7) ||
@@ -193,7 +241,7 @@ static int hsCmpPiece(const EntVariant* hs, const BOARD* bo)
        bo->inhand[ oppo ][ GIN ] != ((q >> 11) & 0x7) ||
        bo->inhand[ oppo ][ KIN ] != ((q >> 14) & 0x7) ||
        bo->inhand[ oppo ][ KAKU ] != ((q >> 17) & 0x3) ||
-       bo->inhand[ oppo ][ HISYA ] != ((q >> 19) & 0x3) ) {
+       bo->inhand[ oppo ][ HISHA ] != ((q >> 19) & 0x3) ) {
     return 1;
   }
 
@@ -217,8 +265,8 @@ static int hsSuperiorBOARD(const EntVariant* hs, const BOARD* bo)
 {
   int next = bo->next;
   int oppo = 1 - bo->next;
-  uint32_t p = hs->packed_inhand[ next ];
-  uint32_t q = hs->packed_inhand[ oppo ];
+  int32_t p = hs->packed_inhand[ next ];
+  int32_t q = hs->packed_inhand[ oppo ];
 
   // 駒箱に駒があることがあるので、両sideを調べる。
   if ( bo->inhand[ next ][ FU ] >= (p & 0x1f) &&
@@ -227,7 +275,7 @@ static int hsSuperiorBOARD(const EntVariant* hs, const BOARD* bo)
        bo->inhand[ next ][ GIN ] >= ((p >> 11) & 0x7) &&
        bo->inhand[ next ][ KIN ] >= ((p >> 14) & 0x7) &&
        bo->inhand[ next ][ KAKU ] >= ((p >> 17) & 0x3) &&
-       bo->inhand[ next ][ HISYA ] >= ((p >> 19) & 0x3) &&
+       bo->inhand[ next ][ HISHA ] >= ((p >> 19) & 0x3) &&
 
        bo->inhand[ oppo ][ FU ] <= (q & 0x1f) &&
        bo->inhand[ oppo ][ KYO ] <= ((q >> 5) & 0x7) &&
@@ -235,7 +283,7 @@ static int hsSuperiorBOARD(const EntVariant* hs, const BOARD* bo)
        bo->inhand[ oppo ][ GIN ] <= ((q >> 11) & 0x7) &&
        bo->inhand[ oppo ][ KIN ] <= ((q >> 14) & 0x7) &&
        bo->inhand[ oppo ][ KAKU ] <= ((q >> 17) & 0x3) &&
-       bo->inhand[ oppo ][ HISYA ] <= ((q >> 19) & 0x3) ) {
+       bo->inhand[ oppo ][ HISHA ] <= ((q >> 19) & 0x3) ) {
     return -1;
   }
   else if ( bo->inhand[ next ][ FU ] <= (p & 0x1f) &&
@@ -244,7 +292,7 @@ static int hsSuperiorBOARD(const EntVariant* hs, const BOARD* bo)
             bo->inhand[ next ][ GIN ] <= ((p >> 11) & 0x7) &&
             bo->inhand[ next ][ KIN ] <= ((p >> 14) & 0x7) &&
             bo->inhand[ next ][ KAKU ] <= ((p >> 17) & 0x3) &&
-            bo->inhand[ next ][ HISYA ] <= ((p >> 19) & 0x3) &&
+            bo->inhand[ next ][ HISHA ] <= ((p >> 19) & 0x3) &&
 
             bo->inhand[ oppo ][ FU ] >= (q & 0x1f) &&
             bo->inhand[ oppo ][ KYO ] >= ((q >> 5) & 0x7) &&
@@ -252,7 +300,7 @@ static int hsSuperiorBOARD(const EntVariant* hs, const BOARD* bo)
             bo->inhand[ oppo ][ GIN ] >= ((q >> 11) & 0x7) &&
             bo->inhand[ oppo ][ KIN ] >= ((q >> 14) & 0x7) &&
             bo->inhand[ oppo ][ KAKU ] >= ((q >> 17) & 0x3) &&
-            bo->inhand[ oppo ][ HISYA ] >= ((q >> 19) & 0x3) ) {
+            bo->inhand[ oppo ][ HISHA ] >= ((q >> 19) & 0x3) ) {
     return 1;
   }
 
@@ -260,32 +308,102 @@ static int hsSuperiorBOARD(const EntVariant* hs, const BOARD* bo)
 }
 
 
+static const HASH* hs_find_entry(int tag, const BOARD* bo, int* size)
+{
+  return static_cast<const HASH*>(
+    tcmapget(ttable_mate[tag], &bo->key, sizeof(bo->key), size) );
+}
+
+
+#ifdef USE_PROOF_PIECES
+/**
+ * 攻め方の持ち駒最小で勝ちの局面にする。
+ * 呼び出した側で boSetToBOARD() すること
+ * @param offense_side 攻め方プレイヤー
+ * @param bo 局面. 更新する
+ *
+ * @return 優越している局面があったとき true
+ */
+bool hs_minimize_proof_pieces(int offense_side, BOARD* bo)
+{
+  bool ret = false;
+
+  int size = 0;
+  const HASH* cntr = hs_find_entry(offense_side, bo, &size);
+  if (!cntr)
+    return false;
+
+  int i;
+  for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
+    const EntVariant* ent = cntr->ent + i;
+
+    if (bo->next == offense_side) {
+      if ( ent->phi == 0 && ent->delta >= +VAL_INF ) {
+        if (hsSuperiorBOARD(ent, bo) < 0) {
+          unpack_inhand(ent, bo);
+          ret = true;
+        }
+      }
+    }
+    else {
+      if ( ent->phi >= +VAL_INF && ent->delta == 0 ) {
+        if ( hsSuperiorBOARD(ent, bo) > 0 ) {
+          unpack_inhand(ent, bo);
+          ret = true;
+        }
+      }
+    }
+  }
+
+  return ret;
+}
+#endif // USE_PROOF_PIECES
+
+
+#ifdef NDEBUG
+  #define assert_tt_put(bo, best_move) (static_cast<void>(0))
+#else
+void assert_tt_put(const BOARD* bo, const TE& best_move)
+{
+  assert(bo);
+
+  if ( te_to(best_move) && te_is_put(best_move) ) {
+    if (bo->inhand[bo->next][te_pie(best_move) & 0x7] <= 0) {
+      printBOARD(bo);
+      printf("add move = %s\n", te_str(best_move, *bo).c_str() );
+      si_abort("inhand is %d\n", bo->inhand[bo->next][te_pie(best_move) & 0x7]);
+    }
+  }
+}
+#endif // !NDEBUG
+
+
 /**
  * 局面を置換表に登録する。詰め将棋用。
  *
- * @param offense_side 攻め方
+ * @param tag 攻め方とレベル
  * @param bo 対象のBOARD
  * @param phi   攻め方ノードでは証明数.
  * @param delta 攻め方ノードでは反証数. (phi, delta) = (0, +INF) で勝ち
  * @param te_count 手数. 深さではない.
  * @param best_move 最善手. 投了はNULL
  */
-void hsPutBOARD( int offense_side, const BOARD* bo, 
-                 int32_t phi, int32_t delta, int te_count, const TE* best_move )
+void hsPutBOARD( int tag, const BOARD* bo, 
+                 int32_t phi, int32_t delta, int te_count,
+                 const TE& best_move )
 {
-  assert( bo != NULL );
+  assert_tt_put(bo, best_move);
 
   int size = 0;
   // エンディアンは考慮しない。
   // TODO: エンディアンの異なるノードが参加するときは要修正
-  const HASH* old_ent = static_cast<const HASH*>(
-       tcmapget(ttable_mate[offense_side], &bo->key, sizeof(bo->key), &size) );
+  const HASH* old_ent = hs_find_entry(tag, bo, &size);
 
   if (!old_ent) {
     HASH n;
     n.key = bo->key;
     n.ent[0] = make_v(bo, phi, delta, te_count, best_move);
-    tcmapput( ttable_mate[offense_side], &bo->key, sizeof(bo->key), 
+    tcmapput( ttable_mate[tag], &bo->key, sizeof(bo->key), 
 	      &n, sizeof(HASH) );
     return;
   }
@@ -307,7 +425,8 @@ void hsPutBOARD( int offense_side, const BOARD* bo,
         // -> 既存のは削除してよい
         // skip
       }
-      else if ( hsSuperiorBOARD(ent, bo) < 0 && (ent->phi == 0 && ent->delta >= VAL_INF) ) {
+      else if ( hsSuperiorBOARD(ent, bo) < 0 && 
+		(ent->phi == 0 && ent->delta >= VAL_INF) ) {
         // 既存のも詰み. 新たに追加する必要なし
         to_add = false;
         new_ent->ent[count++] = *ent;
@@ -326,7 +445,8 @@ void hsPutBOARD( int offense_side, const BOARD* bo,
       if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) < 0 ) {
         // skip
       }
-      else if ( hsSuperiorBOARD(ent, bo) > 0 && (ent->phi >= VAL_INF && ent->delta == 0) ) {
+      else if ( hsSuperiorBOARD(ent, bo) > 0 && 
+		(ent->phi >= VAL_INF && ent->delta == 0) ) {
         // 既存のも不詰み
         to_add = false;
         new_ent->ent[count++] = *ent;
@@ -336,7 +456,7 @@ void hsPutBOARD( int offense_side, const BOARD* bo,
     }
   }
   else {
-    // 探索途中. 持ち駒が一致し、一番深いものを1つだけ残す
+    // 探索途中.
     for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
       const EntVariant* ent = old_ent->ent + i;
 
@@ -371,45 +491,56 @@ void hsPutBOARD( int offense_side, const BOARD* bo,
   // 追加すべきときだけ実際に書き込む
   if (to_add) {
     new_ent->ent[count++] = make_v(bo, phi, delta, te_count, best_move);
-    tcmapput( ttable_mate[offense_side], &bo->key, sizeof(bo->key),
+    tcmapput( ttable_mate[tag], &bo->key, sizeof(bo->key),
               new_ent, HASH_HEADER_SIZE + HASH_ENT_SIZE * count );
 
-    tcmapmove( ttable_mate[offense_side], &bo->key, sizeof(bo->key), false );
+    tcmapmove( ttable_mate[tag], &bo->key, sizeof(bo->key), false );
 
-    if ( tcmaprnum(ttable_mate[offense_side]) > TTABLE_COUNT )
-      tcmapcutfront(ttable_mate[offense_side], TTABLE_COUNT / 3);
+    if ( tcmaprnum(ttable_mate[tag]) > TTABLE_ENT_COUNT )
+      tcmapcutfront(ttable_mate[tag], TTABLE_ENT_COUNT / 3);
   }
   else
-    tcmapmove( ttable_mate[offense_side], &bo->key, sizeof(bo->key), false );
+    tcmapmove( ttable_mate[tag], &bo->key, sizeof(bo->key), false );
 
   free(new_ent);
 }
 
 
-extern long G_COUNT_HASH_CHECK_MATE;
-/* extern long G_COUNT_HASH_NOT_CHECK_MATE; */
+#ifdef NDEBUG
+  #define assert_post_tt_get(bo, best_move) (static_cast<void>(0))
+#else
+void assert_post_tt_get(const BOARD* bo, const TE& best_move) 
+{
+  if ( te_to(best_move) && te_is_put(best_move) ) {
+    if (bo->inhand[bo->next][te_pie(best_move) & 0x7] <= 0) {
+      printBOARD(bo);
+      printf("get best move = %s\n", te_str(best_move, *bo).c_str() );
+      si_abort("inhand is %d\n", bo->inhand[bo->next][te_pie(best_move) & 0x7]);
+    }
+  }
+}
+#endif // !NDEBUG
+
 
 /**
  * 局面の得点 (score) を取り出す。
  *
- * @param offense_side 攻め方
+ * @param tag 攻め方とレベル
  * @param bo 対象のBOARD
  * @param phi 攻め方ノードで証明数.
  * @param delta 攻め方ノードで反証数.
- * @param te 最善手
+ * @param te 最善手. NULLでもよい
  * @param te_count 手数. 深さではない
  *
  * @return 盤面が見つかったら 1 、見つからなかったら 0 を返す。
  */
-int hsGetBOARD( int offense_side, const BOARD* bo, 
-                int* phi, int* delta, TE* te, int te_count )
+int hsGetBOARD( int tag, const BOARD* bo, 
+                int32_t* phi, int32_t* delta, TE* te, int te_count )
 {
-  assert( bo != NULL );
   assert( phi && delta );
 
   int size = 0;
-  const HASH* cntr = static_cast<const HASH*>(
-      tcmapget(ttable_mate[offense_side], &bo->key, sizeof(bo->key), &size) );
+  const HASH* cntr = hs_find_entry(tag, bo, &size);
   if (!cntr)
     return 0;
 
@@ -417,67 +548,96 @@ int hsGetBOARD( int offense_side, const BOARD* bo,
   for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
     const EntVariant* ent = cntr->ent + i;
 
-    if ( ent->phi == 0 && ent->delta >= VAL_INF ) {
+    if ( (ent->phi == 0 && ent->delta >= VAL_INF) || 
+	 (ent->te_count >= te_count && 
+	          ent->phi == 0 && ent->delta >= VAL_MAY_MATCH) ) {
       // 現局面のほうが優越してれば詰み
       if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) < 0 ) {
 	*phi = ent->phi; *delta = ent->delta;
-        if (te)
+        if (te) {
           *te = te_unpack(ent->best_move);
+	  assert_post_tt_get(bo, *te);
+	}
 	return 1;
       }
     }
-    else if ( ent->phi >= VAL_INF && ent->delta == 0 ) {
+    else if ( (ent->phi >= VAL_INF && ent->delta == 0) ||
+	      (ent->te_count >= te_count && 
+	               ent->phi >= VAL_MAY_MATCH && ent->delta == 0) ) {
       // 不詰み
       if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) > 0 ) {
 	*phi = ent->phi; *delta = ent->delta;
-        if (te)
+        if (te) {
           *te = te_unpack(ent->best_move);
+
+	  // 優越関係によっては指せないことがある
+	  if ( te_is_put(*te) && bo->inhand[bo->next][te_pie(*te) & 0x7] <= 0)
+	    te_clear(te);
+
+	  assert_post_tt_get(bo, *te);
+	}
 	return 1;
       }
     }
     else {
+      if ( (ent->phi >= VAL_MAY_MATCH && ent->delta == 0) ||
+           (ent->phi == 0 && ent->delta >= VAL_MAY_MATCH) )
+        continue;
+#if 0
       if (ent->te_count < te_count)
         continue;
+#endif
 
-      // TODO: ここでも優越関係を使える？
       if ( !hsCmpPiece(ent, bo) ) {
 	*phi = ent->phi; *delta = ent->delta;
-        if (te)
+        if (te) {
           *te = te_unpack(ent->best_move);
+	  assert_post_tt_get(bo, *te);
+	}
 	return 1;
       }
     }
-  }
+  } // of for
 
   // 見付からなかった
   return 0;
 }
 
 
-extern long count_entry_hash;
-
 /**
  * min/max検索用にBOARDをハッシュテーブルに登録する。
+ * phi,delta = (+VAL_INF, -VAL_INF) で勝ち
+ *
  * @param bo 対象のBOARD
- * @param score 盤面の評価点数
+ * @param phi 盤面の評価点数
+ * @param delta 局面の評価点数
  * @param te_count 手数. 深さではない
  * @param best_move 最善手. 投了のときはNULL
  */
-void hsPutBOARDMinMax( const BOARD* bo, int32_t score, int te_count, 
-                       const TE* best_move )
+void hsPutBOARDMinMax( const BOARD* bo, int32_t phi, int32_t delta,
+		       int te_count, 
+                       const TE& best_move )
 {
-  assert( bo != NULL );
+  assert_tt_put(bo, best_move);
 
-  // TODO: 優越関係が使えるかな？
   int size = 0;
+#ifdef HASH_ON_FILE
+  HASH* old_ent = static_cast<HASH*>(
+		     tchdbget(ttable, &bo->key, sizeof(bo->key), &size) );
+#else
   const HASH* old_ent = static_cast<const HASH*>(
                 	 tcmapget(ttable, &bo->key, sizeof(bo->key), &size) );
+#endif
 
   if (!old_ent) {
     HASH n;
     n.key = bo->key;
-    n.ent[0] = make_v(bo, score, 0, te_count, best_move);
+    n.ent[0] = make_v(bo, phi, delta, te_count, best_move);
+#ifdef HASH_ON_FILE
+    tchdbput( ttable, &bo->key, sizeof(bo->key), &n, sizeof(HASH) );
+#else
     tcmapput( ttable, &bo->key, sizeof(bo->key), &n, sizeof(HASH) );
+#endif
     return;
   }
 
@@ -488,7 +648,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t score, int te_count,
   bool to_add = true;
   int i;
 
-  if (score >= VAL_INF) {
+  if ( phi >= +VAL_INF && delta <= -VAL_INF ) {
     // 勝ち
     for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
       const EntVariant* ent = old_ent->ent + i;
@@ -497,8 +657,9 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t score, int te_count,
         // より少ない持ち駒
         // skip
       }
-      else if ( hsSuperiorBOARD(ent, bo) < 0 && ent->phi >= VAL_INF ) {
-        // hsPutBOARD() とはここが違うだけ
+      else if ( hsSuperiorBOARD(ent, bo) < 0 && 
+		(ent->phi >= +VAL_INF && ent->delta <= -VAL_INF) ) {
+        // 既存のも勝ち
         to_add = false;
         new_ent->ent[count++] = *ent;
       }
@@ -506,7 +667,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t score, int te_count,
         new_ent->ent[count++] = *ent;
     }
   }
-  else if (score <= -VAL_INF) {
+  else if ( phi <= -VAL_INF && delta >= +VAL_INF ) {
     // 負け
     for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
       const EntVariant* ent = old_ent->ent + i;
@@ -514,7 +675,8 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t score, int te_count,
       if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) < 0 ) {
         // skip
       }
-      else if ( hsSuperiorBOARD(ent, bo) > 0 && ent->phi <= -VAL_INF ) {
+      else if ( hsSuperiorBOARD(ent, bo) > 0 && 
+		(ent->phi <= -VAL_INF && ent->delta >= +VAL_INF) ) {
         to_add = false;
         new_ent->ent[count++] = *ent;
       }
@@ -527,7 +689,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t score, int te_count,
     for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
       const EntVariant* ent = old_ent->ent + i;
 
-      if ( ent->phi >= +VAL_INF ) {
+      if ( ent->phi >= +VAL_INF && ent->delta <= -VAL_INF ) {
         // 既存のエントリが勝ち
         if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) < 0 ) {
           to_add = false;
@@ -536,7 +698,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t score, int te_count,
         else
           new_ent->ent[count++] = *ent;
       }
-      else if (ent->phi <= -VAL_INF) {
+      else if ( ent->phi <= -VAL_INF && ent->delta >= +VAL_INF ) {
         if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) > 0 ) {
           to_add = false;
           new_ent->ent[count++] = *ent;
@@ -555,18 +717,30 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t score, int te_count,
   }
 
   if (to_add) {
-    new_ent->ent[count++] = make_v(bo, score, 0, te_count, best_move);
+    new_ent->ent[count++] = make_v(bo, phi, delta, te_count, best_move);
+#ifdef HASH_ON_FILE
+    tchdbput( ttable, &bo->key, sizeof(bo->key), new_ent,
+              HASH_HEADER_SIZE + HASH_ENT_SIZE * count );
+#else
     tcmapput( ttable, &bo->key, sizeof(bo->key), new_ent, 
 	      HASH_HEADER_SIZE + HASH_ENT_SIZE * count );
 
     tcmapmove( ttable, &bo->key, sizeof(bo->key), false );
 
-    if ( tcmaprnum(ttable) > TTABLE_COUNT )
-      tcmapcutfront(ttable, TTABLE_COUNT / 3);
+    if ( tcmaprnum(ttable) > TTABLE_ENT_COUNT )
+      tcmapcutfront(ttable, TTABLE_ENT_COUNT / 3);
+#endif
   }
-  else
+  else {
+#ifndef HASH_ON_FILE
+    // 移動
     tcmapmove( ttable, &bo->key, sizeof(bo->key), false );
+#endif // !HASH_ON_FILE
+  }
 
+#ifdef HASH_ON_FILE
+  free( old_ent );
+#endif
   free(new_ent);
 }
 
@@ -575,18 +749,25 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t score, int te_count,
  * 盤面の得点 (score) を取りだす。min/max検索用。
  * @param bo 対象のBOARD
  * @param score 得点を格納する変数のポインタ
+ * @param te 見つかった指し手を格納する先
  * @param te_count 手数. 深さではない
  *
  * @return 盤面が見つかったら 1 、見つからなかったら 0 を返す。
  */
-int hsGetBOARDMinMax( const BOARD* bo, int32_t* score, TE* te, int te_count )
+int hsGetBOARDMinMax( const BOARD* bo, PhiDelta* score, TE* te, int te_count )
 {
   assert( bo != NULL );
   assert( score );
 
   int size = 0;
+#ifdef HASH_ON_FILE
+  HASH* cntr = static_cast<HASH*>(
+		     tchdbget(ttable, &bo->key, sizeof(bo->key), &size) );
+#else
   const HASH* cntr = static_cast<const HASH*>(
    		      tcmapget(ttable, &bo->key, sizeof(bo->key), &size) );
+#endif
+
   if (!cntr)
     return 0;
 
@@ -594,37 +775,63 @@ int hsGetBOARDMinMax( const BOARD* bo, int32_t* score, TE* te, int te_count )
   for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
     const EntVariant* ent = cntr->ent + i;
 
-    if ( ent->phi >= +VAL_INF ) {
+    if ( ent->phi >= +VAL_INF && ent->delta <= -VAL_INF ) {
       // 勝ち.
       if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) < 0 ) {
-        *score = ent->phi;
-        if (te)
+        score->phi = ent->phi; score->delta = ent->delta;
+        if (te) {
           *te = te_unpack(ent->best_move);
+	  assert_post_tt_get(bo, *te);
+	}
+#ifdef HASH_ON_FILE
+	free( cntr );
+#endif
         return 1;
       }
     }
-    else if ( ent->phi <= -VAL_INF ) {
+    else if ( ent->phi <= -VAL_INF && ent->delta >= +VAL_INF ) {
       // 負け
       if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) > 0 ) {
-        *score = ent->phi;
-        if (te)
+        score->phi = ent->phi; score->delta = ent->delta;
+        if (te) {
           *te = te_unpack(ent->best_move);
+
+	  // 優越関係によっては指せないことがある
+	  if ( te_is_put(*te) && bo->inhand[bo->next][te_pie(*te) & 0x7] <= 0)
+	    te_clear(te);
+
+	  assert_post_tt_get(bo, *te);
+	}
+#ifdef HASH_ON_FILE
+	free( cntr );
+#endif
         return 1;
       }
     }
     else {
+#if 0
       if (ent->te_count < te_count)
         continue;
+#endif // !USE_ANY_MOVE
 
       if ( !hsCmpPiece(ent, bo) ) {
-        *score = ent->phi;
-        if (te)
+        score->phi = ent->phi; score->delta = ent->delta;
+        if (te) {
           *te = te_unpack(ent->best_move);
+	  assert_post_tt_get(bo, *te);
+	}
+#ifdef HASH_ON_FILE
+	free( cntr );
+#endif
         return 1;
       }
     }
   } // of for
 
+#ifdef HASH_ON_FILE
+  free( cntr );
+#endif
+
   // 見付からなかった
   return 0;
 }
@@ -637,24 +844,15 @@ int hsGetBOARDMinMax( const BOARD* bo, int32_t* score, TE* te, int te_count )
  */
 void bo_add_hash_value(BOARD* bo, Xy xy)
 {
-  int p;
+  PieceKind p;
   
-#ifdef DEBUG
-  assert(bo != NULL);
   assert( 1 <= (xy >> 4) && (xy >> 4) <= 9 );
   assert( 1 <= (xy & 0xF) && (xy & 0xF) <= 9 );
-#endif /* DEBUG */
-  
+
   p = boGetPiece(bo, xy);
 
-  if ( bo->next == SENTE ) {
-    bo->key ^= hashval[ p ][ xy ];
-    bo->hash_all ^= hashval[p][xy];
-  }
-  else {
-    bo->key ^= ( ~ hashval[ p ][ xy ] );
-    bo->hash_all ^= ~ hashval[p][xy];
-  }
+  bo->key ^= hashval[0][xy] ^ hashval[ p ][ xy ];
+  bo->hash_all ^= hashval[0][xy] ^ hashval[p][xy];
 }
 
 
@@ -665,7 +863,7 @@ void bo_add_hash_value(BOARD* bo, Xy xy)
  */
 void bo_sub_hash_value(BOARD* bo, Xy xy)
 {
-  int p;
+  PieceKind p;
   
 #ifdef DEBUG
   assert(bo != NULL);
@@ -675,28 +873,63 @@ void bo_sub_hash_value(BOARD* bo, Xy xy)
   
   p = boGetPiece(bo, xy);
   
-  if ( bo->next == SENTE ) {
-    bo->key ^= hashval[ p ][ xy ];
-    bo->hash_all ^= hashval[p][xy];
-  }
-  else {
-    bo->key ^= ( ~ hashval[ p ][ xy ] );
-    bo->hash_all ^= ~ hashval[p][xy];
-  }
+  bo->key ^= hashval[0][xy] ^ hashval[ p ][ xy ];
+  bo->hash_all ^= hashval[0][xy] ^ hashval[p][xy];
 }
 
 
 /** 使用中のエントリの個数を得る */
 int hs_entry_count(int tag)
 {
-  switch (tag) {
-  case 0:
-    return tcmaprnum(ttable_mate[0]);
-  case 1:
-    return tcmaprnum(ttable_mate[1]);
-  case 2:
+  if (tag < 0) {
+#ifdef HASH_ON_FILE
+    return tchdbrnum( ttable );
+#else
     return tcmaprnum(ttable);
-  default:
-    assert(0);
+#endif
+  }
+  else if (tag < TTABLE_COUNT)
+    return tcmaprnum( ttable_mate[tag] );
+  else {
+    abort();
   }
 }
+
+
+/** 詰め手順を得る */
+void hs_get_best_sequence( int level, const BOARD* bo, MOVEINFO2* seq )
+{
+  assert( level >= -1 && level <= 2 );
+
+  seq->count = 0;
+
+  int32_t phi, delta;
+  TE te_ret;
+
+  int const offense_side = bo->next;
+  BOARD* bo_tmp = bo_dup(bo);
+
+  while ( true && seq->count < 100 ) {
+    int r;
+    te_clear(&te_ret);
+    if (level >= 0)
+      r = hsGetBOARD(level * 2 + offense_side, bo_tmp,
+                     &phi, &delta, &te_ret, 0);
+    else {
+      PhiDelta dmy;
+      r = hsGetBOARDMinMax(bo_tmp, &dmy, &te_ret, 0);
+    }
+
+    if (!r)
+      break;
+
+    mi2Add( seq, te_ret); // 投了も登録する
+    if (!te_to(te_ret) )
+      break;
+
+    boMove_mate(bo_tmp, te_ret);
+  }
+  freeBOARD(bo_tmp);
+}
+
+
