@@ -2,7 +2,7 @@
   Daemonshogi -- a GTK+ based, simple shogi (Japanese chess) program.
 
   Copyright (C) Masahiko Tokita    2002-2005,2009 
-  Copyright (C) Hisashi Horikawa   2008-2009
+  Copyright (C) Hisashi Horikawa   2008-2010
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -82,6 +82,7 @@ const XyDiff normal_to[][8] = {
 };
 
 
+#if 0
 /**
  * 長い攻撃ではない指し手を生成する。飛・角などの短い攻撃も。
  * @param bo 対象のBOARD
@@ -92,26 +93,19 @@ const XyDiff normal_to[][8] = {
 static void append_short_onboard_moves(const BOARD* bo, Xy xy, PieceKind piece, 
                                        MOVEINFO* mi)
 {
-  int i, c;
+  int i;
 
-  TE te;
-  te.fm = xy;
-  te.nari = 0;
-  te.uti = 0;
-
-  int next = (piece & 0x10) == 0x10;
+  int const next = (piece & 0x10) == 0x10;
 
   for (i = 0; i < normal_to_count[piece]; i++) {
-    int vw;
-
-    vw = xy + normal_to[piece][i];
+    Xy const vw = xy + normal_to[piece][i];
 
 #ifdef DEBUG
     assert( -1 <= (vw >> 4) && (vw >> 4) <= 11 );
     assert( 0 <= (vw & 0xF) && (vw & 0xF) <= 10 );
 #endif /* DEBUG */
 
-    c = boGetPiece(bo, vw);
+    PieceKind const c = boGetPiece(bo, vw);
     if (c == WALL)
       continue;
 
@@ -120,81 +114,10 @@ static void append_short_onboard_moves(const BOARD* bo, Xy xy, PieceKind piece,
 	continue;
     }
 
-    te.to = vw;
-    miAddWithNari(bo, mi, &te, next);
-  }
-}
-
-
-/**
- * 長い攻撃以外の利きを付ける.
- * @param bo 対象のBOARD
- * @param xy 場所
- * @param piece 駒種. 成り、プレイヤーを含む
- */
-static void movetoNormal2(BOARD* bo, Xy xy, PieceKind piece)
-{
-  assert( bo->board[xy] );
-  assert( bo->board[xy] == piece );
-
-  int side = ((piece & 0x10) >> 4);
-  int n = bo->noBoard[xy];
-  int i;
-
-  for (i = 0; i < normal_to_count[piece]; i++) {
-    PieceKind c;
-    Xy vw;
-
-    vw = xy + normal_to[piece][i];
-    c = bo->board[vw];
-    if (c == WALL)
-      continue;
-
-    bo->piece_kiki_count[n]++; // 壁以外はすべて効かす
-    bo->short_attack[side][vw].from[ bo->short_attack[side][vw].count++ ] = xy;
-  }
-}
-
-
-/** 配列から一致するものを削除する */
-static void erase(int16_t* begin, int16_t* end, int16_t value)
-{
-  int16_t* p;
-  for (p = begin; p != end; p++) {
-    if (*p == value) {
-      *p = *(end - 1);
-      *(end - 1) = 0;
-      return;
-    }
-  }
-}
-
-
-/** 短い利きを取り除く*/
-static void remove_short_attack( BOARD* bo, Xy xy )
-{
-  assert( bo->board[xy] );
-
-  PieceKind piece = bo->board[xy];
-  int n = bo->noBoard[xy];
-  int side = getTeban(bo->board[xy]);
-  int i;
-
-  for (i = 0; i < normal_to_count[piece]; i++) {
-    Xy vw;
-
-    vw = xy + normal_to[piece][i];
-    if ( bo->board[vw] == WALL )
-      continue;
-
-    erase( bo->short_attack[side][vw].from,
-           bo->short_attack[side][vw].from + bo->short_attack[side][vw].count,
-           xy );
-    bo->short_attack[side][vw].count--;
-
-    bo->piece_kiki_count[n]--;
+    miAddWithNari( mi, *bo, xy, vw );
   }
 }
+#endif // 0
 
 
 /** remoteOhte() でも利用する */
@@ -206,7 +129,7 @@ const int remote_to_count[] = {
   0, 0, 0, 0, 0, 4, 4, 0,
 };
 
-const int remote_to[][8] = {
+const XyDiff remote_to[][8] = {
   { 0, 0, 0, 0, 0, 0, 0, 0, }, /* dummy */
 
   /* sente */
@@ -255,29 +178,20 @@ const int remote_to[][8] = {
  * @param xy 駒の座標
  * @param next 駒の持ち主
  */
-static void movetoLong(MOVEINFO* mi, const BOARD* bo, Xy xy, int next)
+static void movetoLong(MOVEINFO* mi, const BOARD* bo, Xy const xy, int next)
 {
-  int p, i;
-  int c;
-  int vw;
-  PieceKind piece = boGetPiece(bo, xy);
-  TE te;
-
-  assert( bo != NULL );
   assert( 1 <= (xy & 0xf) && (xy & 0xf) <= 9 );
   assert( 1 <= (xy >> 4) && (xy >> 4) <= 9 );
   assert( bo->board[xy] != 0 );
-  assert( next == 0 || next == 1 );
-  assert( mi != NULL );
+  assert( getTeban(bo->board[xy]) == bo->next );
 
-  p = piece & 0x0F;
+  int i;
+  PieceKind piece = boGetPiece(bo, xy);
+  // PieceKind p = piece & 0x0F;
 
-  te.fm = xy;
-  te.nari = 0;
-  te.uti = 0;
   for (i = 0; i < remote_to_count[piece]; i++) {
-    // こちらでもpinされていないか確認しておく
 #ifdef USE_PIN
+    // こちらでもpinされていないか確認しておく
     if (bo->pinned[xy]) {
       if (bo->pinned[xy] != remote_to[piece][i] &&
            bo->pinned[xy] != -remote_to[piece][i])
@@ -286,11 +200,11 @@ static void movetoLong(MOVEINFO* mi, const BOARD* bo, Xy xy, int next)
 #endif
 
     bool flag = false;
-    vw = xy;
+    Xy vw = xy;
     while (1) {
       vw += remote_to[piece][i];
 
-      c = boGetPiece(bo, vw);
+      PieceKind const c = boGetPiece(bo, vw);
       if (c == WALL)
         break;
 
@@ -301,8 +215,8 @@ static void movetoLong(MOVEINFO* mi, const BOARD* bo, Xy xy, int next)
         flag = true;
       }
 
-      te.to = vw;
-      miAddWithNari(bo, mi, &te, next);
+      // te.to = vw;
+      miAddWithNari( mi, *bo, xy, vw );
 
       if (flag)
         break;
@@ -317,39 +231,110 @@ static void movetoLong(MOVEINFO* mi, const BOARD* bo, Xy xy, int next)
  * @param xy 駒の場所
  * @param mi 結果を格納するMOVEINFO
  */
-void moveto(const BOARD* bo, int xy, MOVEINFO* mi)
+void mi_append_moveto(const BOARD* bo, Xy xy, MOVEINFO* mi)
 {
-  PieceKind piece;
-  int p;
-
-  assert( bo != NULL );
   assert( 1 <= (xy & 0x0f) && (xy & 0x0f) <= 9 );
   assert( 1 <= (xy >> 4) && (xy >> 4) <= 9 );
-  assert( bo->board[xy] != 0);
-  assert(bo->next == 0 || bo->next == 1);
-  assert( mi != NULL );
+  assert( bo->board[xy] != 0 );
+  assert( getTeban(bo->board[xy]) == bo->next );
 
-  // mi->count = 0;
-
-  piece = bo->board[ xy ];
-  p = piece & 0x0F;
-  if ( p == OH )
-    append_king_moves( bo, mi, xy, bo->next );
+  PieceKind const piece = bo->board[ xy ];
+#if 0
+  if ( (piece & 0xf) == OH )
+    mi_append_king_moves( bo, mi, xy, bo->next );
   else
     append_short_onboard_moves( bo, xy, piece, mi );
 
-  if (p == KAKU || p == HISYA || p == UMA || p == RYU || p == KYO)
+  if ( is_long_piece(piece) )
     movetoLong(mi, bo, xy, bo->next);
+#else
+  // 展開してみる   2009.12 多少だが効果あり
+  switch (piece) {
+  case FU:
+    miAddWithNari( mi, *bo, xy, xy - 16 );
+    break;
+  case KYO: case KYO + 0x10:
+  case KAKU: case KAKU + 0x10:
+  case HISHA: case HISHA + 0x10:
+    movetoLong( mi, bo, xy, bo->next );
+    break;
+  case KEI:
+    miAddWithNari( mi, *bo, xy, xy - 33 );
+    miAddWithNari( mi, *bo, xy, xy - 31 );
+    break;
+  case GIN:
+    miAddWithNari( mi, *bo, xy, xy - 16 );
+    miAddWithNari( mi, *bo, xy, xy - 17 );
+    miAddWithNari( mi, *bo, xy, xy + 15 );
+    miAddWithNari( mi, *bo, xy, xy + 17 );
+    miAddWithNari( mi, *bo, xy, xy - 15 );
+    break;
+  case KIN:
+  case TOKIN: case NARIKYO: case NARIKEI: case NARIGIN:
+    miAddWithNari( mi, *bo, xy, xy - 16 );
+    miAddWithNari( mi, *bo, xy, xy - 17 );
+    miAddWithNari( mi, *bo, xy, xy - 1 );
+    miAddWithNari( mi, *bo, xy, xy + 16 );
+    miAddWithNari( mi, *bo, xy, xy + 1 );
+    miAddWithNari( mi, *bo, xy, xy - 15 );
+    break;
+  case OH:
+  case OH + 0x10:
+    mi_append_king_moves( bo, mi, xy, bo->next );
+    break;
+  case UMA:
+  case UMA + 0x10:
+    miAddWithNari( mi, *bo, xy, xy - 16 );
+    miAddWithNari( mi, *bo, xy, xy - 1 );
+    miAddWithNari( mi, *bo, xy, xy + 16 );
+    miAddWithNari( mi, *bo, xy, xy + 1 );
+    movetoLong( mi, bo, xy, bo->next );
+    break;
+  case RYU:
+  case RYU + 0x10:
+    miAddWithNari( mi, *bo, xy, xy - 17 );
+    miAddWithNari( mi, *bo, xy, xy + 15 );
+    miAddWithNari( mi, *bo, xy, xy + 17 );
+    miAddWithNari( mi, *bo, xy, xy - 15 );
+    movetoLong( mi, bo, xy, bo->next );
+    break;
+  case FU + 0x10:
+    miAddWithNari( mi, *bo, xy, xy + 16 );
+    break;
+  case KEI + 0x10:
+    miAddWithNari( mi, *bo, xy, xy + 33 );
+    miAddWithNari( mi, *bo, xy, xy + 31 );
+    break;
+  case GIN + 0x10:
+    miAddWithNari( mi, *bo, xy, xy + 16 );
+    miAddWithNari( mi, *bo, xy, xy + 17 );
+    miAddWithNari( mi, *bo, xy, xy - 15 );
+    miAddWithNari( mi, *bo, xy, xy - 17 );
+    miAddWithNari( mi, *bo, xy, xy + 15 );
+    break;
+  case KIN + 0x10:
+  case TOKIN + 0x10: case NARIKYO + 0x10: case NARIKEI + 0x10: case NARIGIN + 0x10:
+    miAddWithNari( mi, *bo, xy, xy + 16 );
+    miAddWithNari( mi, *bo, xy, xy + 17 );
+    miAddWithNari( mi, *bo, xy, xy + 1 );
+    miAddWithNari( mi, *bo, xy, xy - 16 );
+    miAddWithNari( mi, *bo, xy, xy - 1 );
+    miAddWithNari( mi, *bo, xy, xy + 15 );
+    break;
+  default:
+    abort();
+  }
+#endif
 }
 
 
 /**
- * 打ち歩詰めしていないか
+ * 指し手が打ち歩詰めになるか
  * @param bo 歩を打つ前の局面
  * @param te 歩を打つ手
  * @return 打ち歩詰めのときtrue
  */
-static bool is_uchifuzume(const BOARD* bo, const TE& te)
+static bool te_is_uchifuzume(const BOARD* bo, const TE& te)
 {
   int defense;
   Xy defense_king;
@@ -358,16 +343,16 @@ static bool is_uchifuzume(const BOARD* bo, const TE& te)
   if (bo->next) {
     // 後手が攻め
     defense = 0;
-    defense_king = te.to + 16;
+    defense_king = te_to(te) + 16;
   }
   else {
     // 先手が攻め
     defense = 1;
-    defense_king = te.to - 16;
+    defense_king = te_to(te) - 16;
   }
 
-  // 歩に攻め方の利きが付いてなければ取れる
-  if ( bo_attack_count(bo, 1 - defense, te.to) == 0 )
+  // 歩に攻め方の利きが付いてなければ玉で取れる
+  if ( bo_attack_count(bo, 1 - defense, te_to(te)) == 0 )
     return false;
 
   // 受け玉の周りに攻撃側の利きのないところがあればok
@@ -382,36 +367,46 @@ static bool is_uchifuzume(const BOARD* bo, const TE& te)
   }
 
   // 受け側の駒で取れるか (短いの)
-  for (i = 0; i < bo->short_attack[defense][te.to].count; i++) {
-    Xy defense_from = bo->short_attack[defense][te.to].from[i];
+  for (i = 0; i < bo->short_attack[defense][te_to(te)].count; i++) {
+    Xy defense_from = bo->short_attack[defense][te_to(te)].from[i];
     if ( (bo->board[defense_from] & 0xf) == OH )
       continue;
 #ifdef USE_PIN
-    // これは厳密ではない。
+    // pinされているときは決して取れない
     if (!bo->pinned[defense_from])
       return false;
 #else
+    // これは厳密ではない。
     if (bo->long_attack[1 - defense][defense_from].count == 0)
       return false;
 #endif // USE_PIN
   }
 
   // 受け側の駒で取れるか (長いの)
-  for (i = 0; i < bo->long_attack[defense][te.to].count; i++) {
-    Xy defense_from = bo->long_attack[defense][te.to].from[i];
+  for (i = 0; i < bo->long_attack[defense][te_to(te)].count; i++) {
+    Xy defense_from = bo->long_attack[defense][te_to(te)].from[i];
 
 #ifdef USE_PIN
-    // これは厳密ではない。
+    // pinされているときは決して取れない
     if (!bo->pinned[defense_from])
       return false;
 #else
+    // これは厳密ではない。
     if (bo->long_attack[1 - defense][defense_from].count == 0)
       return false;
 #endif // USE_PIN
   }
-  
+
+#ifdef USE_PIN
+  // 歩で遮られることはない -> 受けなし
+  if ( !bo->long_attack[1 - defense][te_to(te) - 1].count &&
+       !bo->long_attack[1 - defense][te_to(te) + 1].count )
+    return true;
+#endif
+
+  // 歩を打つことで攻め方の長い攻撃が遮られることがある
   // 実際に動かしてみる
-  BOARD* tmp_bo = boCopy(bo);
+  BOARD* tmp_bo = bo_dup(bo);
   boMove_mate( tmp_bo, te );
 
   MOVEINFO tmp_mi;
@@ -424,6 +419,34 @@ static bool is_uchifuzume(const BOARD* bo, const TE& te)
 }
 
 
+/**
+ * ルール上正しい指し手かチェックする。
+ * canvas.cc から呼び出される. この関数は重い
+ *
+ * @param bo 対象のBOARD
+ * @param te 調べる一手
+ * @return 正しければ true を返す。正しくなければ false を返す。
+ */
+bool te_is_valid( const BOARD* bo, const TE& te)
+{
+  MOVEINFO mi;
+
+  make_moves(bo, &mi);  // 打ち歩詰めが含まれる
+  if ( !mi_includes(&mi, te) )
+    return false;
+
+  if ( te_is_put(te) && te_pie(te) == FU ) {
+    if ( (bo->next == SENTE && te_to(te) == bo->king_xy[GOTE] + 16) ||
+	 (bo->next == GOTE && te_to(te) == bo->king_xy[SENTE] - 16) ) {
+      if ( te_is_uchifuzume(bo, te) )
+	return false;
+    }
+  }
+
+  return true;
+}
+
+
 /**
  * ある駒を打つ手を生成.
  * utiOhte() から呼び出される. 打ち歩詰めになったりするときは追加しない。
@@ -434,65 +457,69 @@ static bool is_uchifuzume(const BOARD* bo, const TE& te)
  */
 void append_a_put_move( const BOARD* bo, MOVEINFO* mi, PieceKind pie, Xy xy )
 {
-  TE te;
-
-  te.fm = 0;
-  te.to = xy;
-  te.uti = pie;
-  te.nari = false;
-  te.hint = 0;
+  TE te = te_make_put(xy, pie);
 
   if (bo->next == 0) {
     // 先手
-    if (pie == FU) {
+    switch (pie) {
+    case FU:
       if (checkFuSente(bo, xy & 0xf) || xy <= 0x19)
         return;
-
+#if 0
       // 打ち歩詰め？
       if (bo->board[xy - 16] == (OH + 0x10)) {
         if (is_uchifuzume(bo, te))
           return;
       }
-    }
-    else if (pie == KYO) {
+#endif // 0
+      break;
+    case KYO:
       if (xy <= 0x19)
         return;
-    }
-    else if (pie == KEI) {
+      break;
+    case KEI:
       if (xy <= 0x29)
         return;
+      break;
+    default:
+      break;
     }
   }
   else {
     // 後手
-    if (pie == FU) {
+    switch (pie) {
+    case FU:
       if (checkFuGote(bo, xy & 0xf) || xy >= 0x91)
         return;
-
+#if 0
       if (bo->board[xy + 16] == OH) {
         if (is_uchifuzume(bo, te))
           return;
       }
-    }
-    else if (pie == KYO) {
+#endif // 0
+      break;
+    case KYO:
       if (xy >= 0x91)
         return;
-    }
-    else if (pie == KEI) {
+      break;
+    case KEI:
       if (xy >= 0x81)
         return;
+      break;
+    default:
+      break;
     }
   }
 
-  miAdd(mi, &te);
+  miAdd(mi, te);
 }
 
 
 /**
  * ある場所に打つ手をすべて生成
- * search_068() と共通
+ * 2009.11 ループの内部でこの関数を呼び出すと遅い. 
  */
-void append_put_moves(const BOARD* bo, Xy xy, MOVEINFO* mi)
+void mi_append_put_moves(const BOARD* bo, Xy xy, MOVEINFO* mi)
 {
   assert( xy >= 0x11 && xy <= 0x99 );
   assert( !bo->board[xy] );
@@ -509,264 +536,42 @@ void append_put_moves(const BOARD* bo, Xy xy, MOVEINFO* mi)
 
 /**
  * 局面で着手可能な指し手を生成する
+ * 打ち歩詰めの手も生成する
+ *
  * @param bo 局面
  * @param mi 手一覧
  */
 void make_moves(const BOARD* bo, MOVEINFO* mi)
 {
-  int x, y;
+  int i;
+  PieceKind inhand_pie[8];
 
   // 王手を掛けられているか
-  Xy king_xy = bo->code_xy[bo->next + 1][0];
-  if ( king_xy && bo_attack_count(bo, 1 - bo->next, king_xy) ) {
-    uke(bo, mi );
+  if ( bo_is_checked(bo, bo->next) ) {
+    uke( bo, mi );
     return;
   }
 
   mi->count = 0;
 
-  for (y = 1; y <= 9; y++) {
-    for (x = 1; x <= 9; x++) {
-      int xy = (y << 4) + x;
+  // あらかじめ持ち駒セットを作る. 2009.11 効果あり
+  int p = 0;
+  for (i = 1; i <= 7; i++) {
+    if ( bo->inhand[bo->next][i] > 0 )
+      inhand_pie[p++] = i;
+  }
+  inhand_pie[p] = 0;
 
-      if (bo->board[xy]) {
-        // xyから移動する手
-        if ( getTeban(bo->board[xy]) == bo->next ) 
-          moveto(bo, xy, mi);
-      }
-      else {
-        // 打つ手
-        append_put_moves(bo, xy, mi);
-      }
+  for (i = 1; i <= 81; i++) {
+    if ( bo->board[arr_xy[i]] ) {
+      // xyから移動する手
+      if ( getTeban(bo->board[arr_xy[i]]) == bo->next ) 
+	mi_append_moveto(bo, arr_xy[i], mi);
+    }
+    else {
+      // 打つ手
+      for (p = 0; inhand_pie[p]; p++)
+	append_a_put_move( bo, mi, inhand_pie[p], arr_xy[i] );
     }
   }
 }
-
-
-/**
- * 長い攻撃による利きを付ける.
- * @param bo 対象のBOARD
- * @param xy 駒の座標
- */
-void bo_add_long_attack(BOARD* bo, Xy xy)
-{
-  assert( bo != NULL );
-  assert( bo->board[xy] );
-
-  int p, i;
-  int side = getTeban(bo->board[xy]);
-  int vw;
-  PieceKind piece;
-  
-  int n = bo->noBoard[xy];
-  piece = bo->board[ xy ];
-  p = piece & 0x0F;
-
-  for (i = 0; i < remote_to_count[piece]; i++) {
-    vw = xy;
-    while (1) {
-      vw += remote_to[piece][i];
-
-      if ( bo->board[ vw ] == WALL )
-        break;
-
-      // mi->to[mi->count++] = vw;
-      bo->piece_kiki_count[n]++;
-      bo->long_attack[side][vw].from[ bo->long_attack[side][vw].count++ ] = xy;
-
-      if ( bo->board[ vw ] ) {
-#ifdef USE_PIN
-        if ((bo->board[vw] & 0x10) != (next << 4)) { 
-          // 相手の駒 ... pin情報を更新
-	  int vw2 = vw;
-          do {
-	    vw2 += remote_to[piece][i];
-	  } while (!bo->board[vw2]);
-
-	  if (bo->board[vw2] == OH + ((1 - next) << 4) ) 
-	    bo->pinned[vw] = remote_to[piece][i];
-	}
-#endif
-        break;
-      }
-    }
-  }
-}
-
-
-/** 長い利きを取り除く */
-void bo_remove_long_attack(BOARD* bo, Xy xy, PieceKind piece)
-{
-  assert( bo->board[xy] );
-
-  int side = getTeban(bo->board[xy]);
-  int i;
-  int* pie_kiki_count = &bo->piece_kiki_count[ bo->noBoard[xy] ];
-
-  for (i = 0; i < remote_to_count[piece]; i++) {
-    Xy vw = xy;
-    while (1) {
-      vw += remote_to[piece][i];
-
-      if ( bo->board[ vw ] == WALL )
-        break;
-
-      erase( bo->long_attack[side][vw].from,
-             bo->long_attack[side][vw].from + bo->long_attack[side][vw].count,
-             xy );
-      bo->long_attack[side][vw].count--;
-
-      // erase( pie_kiki->to, pie_kiki->to + pie_kiki->count, vw);
-      (*pie_kiki_count)--;
-
-      if (bo->board[vw]) {
-#ifdef USE_PIN
-        ...
-#endif
-        break;
-      }
-    }
-  }
-}
-
-
-/**
- * xy にある駒の利き (短いのも長いのも) を取り除く。
- * @param bo BOARD
- * @param xy 座標
- */
-void boPopToBoard(BOARD* bo, Xy xy) 
-{
-  // int fm, to;
-  int n;
-  int next;
-  // KikiInfo* mi;
-  
-  assert( bo != NULL );
-  assert( 1 <= (xy&0x0f) && (xy&0x0f) <= 9 );
-  assert( 1 <= (xy>>4) && (xy>>4) <= 9 );
-  assert( bo->board[ xy ] != 0 );
-  
-  n = bo->noBoard[ xy ];
-  // mi = boGetCodeTo(bo, n);
-  next = getTeban(bo->board[ xy ]);
-
-  remove_short_attack( bo, xy );
-
-  int p = bo->board[xy] & 0xf;
-  if (p == KAKU || p == HISYA || p == UMA || p == RYU || p == KYO)
-    bo_remove_long_attack( bo, xy, bo->board[xy] );
-
-  assert( bo->piece_kiki_count[n] == 0 );
-}
-
-
-/**
- * xyにある駒の利き情報 kiki, toBoard3 を更新する.
- * @param bo 対象のBOARD
- * @param xy 駒の場所
- */
-void bo_movetoKiki(BOARD* bo, Xy xy)
-{
-  PieceKind piece;
-  int next, p;
-  int nn;
-  // KikiInfo* mi;
-
-  // assert( bo != NULL );
-  assert( 1 <= (xy & 0x0f) && (xy & 0x0f) <= 9);
-  assert( 1 <= (xy >> 4) && (xy >> 4) <= 9 );
-  assert(bo->next == 0 || bo->next == 1);
-  assert(bo->board[xy] != 0);
-
-  nn = bo->noBoard[xy];
-  // mi = &bo->kiki[nn];
-  bo->piece_kiki_count[nn] = 0;
-  
-  piece = bo->board[ xy ];
-  next = (0x10 & piece) ? GOTE : SENTE;
-
-  movetoNormal2(bo, xy, piece);
-
-  p = piece & 0x0F;
-  if (p == KAKU || p == HISYA || p == UMA || p == RYU || p == KYO)
-    bo_add_long_attack(bo, xy);
-}
-
-
-/**
- * xy に動ける next 手番の駒を探して MOVEINFO に te を追加する。
- * ただし王は除く。受け/王手生成用。
- * @param bo 対象のBOARD
- * @param mi MOVEINFO
- * @param xy 場所
- * @param next 次の手番
- */
-void moveto2(const BOARD* bo, MOVEINFO* mi, Xy xy, int next)
-{
-  assert( !bo->board[xy] || getTeban(bo->board[xy]) != next );
-
-  TE te;
-  int i, p;
-
-  // mi->count = 0;
-
-  te.nari = 0;
-  te.uti = 0;
-  te.to = xy;
-
-  // 短いの
-  for (i = 0; i < bo->short_attack[ next ][ xy ].count; i++) {
-    te.fm = bo->short_attack[ next ][ xy ].from[i];
-
-    p = bo->board[ te.fm ];
-    if ( p == OH || p == (OH + 0x10) )
-      continue;
-
-    miAddWithNari(bo, mi, &te, next);
-  }
-
-  // 長いの
-  for (i = 0; i < bo->long_attack[ next ][ xy ].count; i++) {
-    te.fm = bo->long_attack[ next ][ xy ].from[i];
-
-    miAddWithNari(bo, mi, &te, next);
-  }
-}
-
-
-/**
- * xy に動ける next 手番の駒を探して MOVEINFO に te を記録する。
- * mi をクリアしないで手を追加する。自分の駒がある場合は mi に記録しない。
- * 取る手の生成用
- * @param bo 対象のBOARD
- * @param mi MOVEINFO
- * @param xy 場所
- * @param next 次の手番
- */
-void moveto2_add(const BOARD* bo, MOVEINFO* mi, Xy xy, int next)
-{
-  TE te;
-  int i, p2;
-  
-  p2 = boGetPiece(bo, xy);
-  if ( p2 != 0 && getTeban(p2) == next ) 
-    return;
-
-  te.nari = 0;
-  te.uti = 0;
-  te.to = xy;
-
-  // 短いの
-  for (i = 0; i < bo->short_attack[ next ][ xy ].count; i++) {
-    te.fm = bo->short_attack[ next ][ xy ].from[i];
-    
-    miAddWithNari(bo, mi, &te, next);
-  }
-
-  for (i = 0; i < bo->long_attack[ next ][ xy ].count; i++) {
-    te.fm = bo->long_attack[ next ][ xy ].from[i];
- 
-    miAddWithNari(bo, mi, &te, next);
-  }
-}
